﻿//index.html页面js：地图相关
//创建：汪浩（qq :346819890） 2016-11-10



//地图对象
var map;

//统一全局变量
var gisdata = {
    config: null,       //config配置信息
    layerVec: null,     //矢量图底图
    layerImg: null,     //卫星图底图
    location: null,     //记录当前定位坐标
    overlaymap: {}
};

//余杭地图CRS 
var yhcrs = new L.Proj.CRS('EPSG:4490', '+proj=longlat +ellps=GRS80 +no_defs', {
    origin: [-180, 90],
    resolutions: [1.40625,
        0.703125, 0.3515625, 0.17578125, 0.087890625,
        0.0439453125, 0.02197265625, 0.010986328125, 0.0054931640625,
        0.00274658203125, 0.001373291015625,
        6.866455206208891E-4, 3.4332276031044456E-4, 1.7166138015522228E-4,
        8.583069007761114E-5, 4.291534503880557E-5,
        2.1457672519402785E-5, 1.0728836259701392E-5, 5.364418129850696E-6,
        2.682209064925349E-6, 1.3411045324626745E-6],
    bounds: L.bounds([119.53125, 29.53125], [120.9375, 30.9375])
});



//加载配置信息
function initMapForConfig(configdata) {
    //赋值
    gisdata.config = configdata;
    gisdata.centerAutoLevel = configdata.centerAutoLevel;


    //地图底图
    map = L.map('map', {
        crs: yhcrs,
        center: configdata.center,
        zoom: configdata.zoom,
        minZoom: configdata.minZoom,
        maxZoom: configdata.maxZoom,
        zoomControl: false,
        attributionControl: false
    });

    //地图底图 
    gisdata.layerVec = getBaseLayer(configdata.basemap.vec);
    gisdata.layerImg = getBaseLayer(configdata.basemap.img);
    gisdata.layerVec.addTo(map);

    //其它地图图层
    var overlayMaps = {};
    if (configdata.overlaymap) {
        $.each(configdata.overlaymap, function (index, item) {
            var layer = createLayerForConfig(item);
            if (layer == null) return;

            //是否显示
            if (item.visible) layer.addTo(map);

            overlayMaps[item.name] = layer;
            var inhtml = '<li><span class="fl mask_03">' + item.name + '</span><span class="switch' + (item.visible ? ' open' : '') + '"><i></i></span></li>';
            $("#layer_manager").append(inhtml);
        });
    }
    gisdata.overlaymap = overlayMaps;
    //余杭区边界
    if (configdata.xzqh) {
        var polyline = L.polyline(configdata.xzqh, { color: '#63b805', weight: 1.5 }).addTo(map);
 
        fitBounds(polyline.getBounds());
        gisdata.overlaymap["余杭边界"] = polyline;
    }
    bindLayerManEvent();

    //地图控件
    var zoomControl = L.control.zoom({ position: "bottomleft" })
    map.addControl(zoomControl);
    var navBar = new L.Control.NavBar({ position: 'bottomleft' });
    map.addControl(navBar);

    //定位到当前位置
    map.on('locationfound', onLocationFound);
    map.on('locationerror', onLocationError);
    locationThis();

 
    //默认值
    gisdata.location = L.latLng(configdata.center[0], configdata.center[1]);
}

function goHome() {
    fitBounds(gisdata.overlaymap["余杭边界"].getBounds()); 
}



//获取基础底图
function getBaseLayer(data, errorTileUrl) {
    var arrVec = [];
    for (var index = 0; index < data.length; index++) {
        var item = data[index];
        if (item.hasOwnProperty("visible") && !item.visible) continue;

        var layer = createLayerForConfig(item, errorTileUrl);
        if (layer == null) continue;

        arrVec.push(layer);
    }
    var layer = L.layerGroup(arrVec);
    layer.name = "底图";
    layer.type = "layerGroup";
    return layer;
}

//创建图层
function createLayerForConfig(item, errorTileUrl) {

    var layer;
    switch (item.type) {
        case "tdt"://国家天地图
            layer = L.tileLayer.tdt({
                layer: item.layer,
                minZoom: item.minZoom, maxZoom: item.maxZoom,
                errorTileUrl: errorTileUrl || 'resources/img/tile/null.png'
            });
            break;
        case "yuhang"://余杭天地图 
            layer = L.tileLayer.wms(item.url, {
                layers: item.layers,
                minZoom: item.minZoom, maxZoom: item.maxZoom,
                transparent: true,
                format: 'image/png',
                errorTileUrl: errorTileUrl || 'resources/img/tile/null.png'
            });
            break;
        case "feature"://arcgis   feature
            layer = L.esri.featureLayer({
                url: item.url,
                opacity: 1,
                where: item.definitionexpression
            });
            break;
        case "tiled"://arcgis tiled
            layer = L.esri.tiledMapLayer({
                url: item.url,
                opacity: 1
            });
            break;
        case "dynamic"://arcgis dynamic 
            layer = L.esri.dynamicMapLayer(item);
            if (item.isPopup) {
                layer.bindPopup(function (error, featureCollection, response) {
                    if (error != null && error.code > 0) {
                        showMsg(error.message);
                        return false;
                    }
                    if (featureCollection.features.length == 0)
                        return false;

                    var popup_columns = gisdata.config.popup_columns;

                    var inhtml = "";
                    var feature = featureCollection.features[0];
                    for (var i in feature.properties) {
                        var showval = $.trim(feature.properties[i]);
                        if (showval == null || showval == '' || showval == '0' || showval.length == 0) continue;

                        var col = popup_columns[i];
                        if (col == null) col = i;
                        if (!isChinese(col)) continue;

                        if (i == "Shape_Area") {
                            showval = Number(showval);
                            if (showval < 1)
                                showval = (showval * 1000000).toFixed(2) + ' 平方米';
                            else
                                showval = (showval).toFixed(2) + ' 平方千米';
                        }

                        inhtml += col + ":" + showval + "<br/>";
                    }
                    return inhtml;
                });
            }
            break;

        default:
            console.log("config配置图层未处理:" + item);
            break;
    }

    if (layer != null) {
        layer.name = item.name;
        layer.type = item.type;
    }
    return layer;
}

function locationThis() {
    map.locate({ setView: true, maxZoom: 16 });
}


//地图定位当前位置事件
function onLocationFound(e) {
    gisdata.location = e.latlng;

    var radius = e.accuracy / 2;
    if (radius > 10000)
        radius = 10000;

    var _mark = L.marker(e.latlng).addTo(map);//.bindPopup("定位精度" + Math.floor(radius) + "m").openPopup();
    var _circle = L.circle(e.latlng, { radius: radius }).addTo(map);

    setTimeout(function () {
        _mark.remove();
        _circle.remove();
    }, 3000)


}

function onLocationError(e) {
    //余杭区政府所在位置(默认值)
    gisdata.location = L.latLng(30.420440, 120.295748);

    showMsg("获取所在位置失败：" + e.message);
}


////地图定位到指定区域方法 
function fitExtent(xmin, ymin, xamx, ymax) {
    var southWest = L.latLng(ymin, xmin);
    var northEast = L.latLng(ymax, xamx);
    var bounds = L.latLngBounds(southWest, northEast);
    fitBounds(bounds);
}
function fitBounds(bounds) {
    map.fitBounds(bounds);
}
 


//图层控制的相关事件
function bindLayerManEvent() {
    $("#btn_layer_vec").click(function (event) {
        $("#layer_manager_view").hide();
        $("#btn_layer_img").removeClass("native");
        $("#btn_layer_vec").addClass("native");

        map.removeLayer(gisdata.layerImg);
        map.addLayer(gisdata.layerVec);
    });
    $("#btn_layer_img").click(function (event) {
        $("#layer_manager_view").hide();
        $("#btn_layer_vec").removeClass("native");
        $("#btn_layer_img").addClass("native");

        map.removeLayer(gisdata.layerVec);
        map.addLayer(gisdata.layerImg);
    });

    //图层切换显示按钮
    $(".switch").on("click", function () {
        var $this = $(this);

        var _visible = $this.hasClass("open");
        if (_visible)
            $this.removeClass("open");
        else
            $this.addClass("open");

        var name = $this.parent().children("span")[0].innerText;
        changeLayerVisile(name, !_visible);

    });
}
//扩展图层的显示隐藏切换
function changeLayerVisile(name, visible) {
    var layer;
    switch (name) {
        default:
            layer = gisdata.overlaymap[name];
            break;
        case "热力图":
            workHeat.changeVisible(visible);
            break;
        case "房产":
            workFangChan.changeVisible(visible);
            break;
    }
    if (layer == null)
        return;

    if (visible)
        map.addLayer(layer);
    else
        map.removeLayer(layer);
}


//自定义控件父类
L.MuyaoControl = L.Control.extend({
    options: {
        position: 'topleft',
        callback: null,
        title: '',
        innerHTML: ''
    },
    onAdd: function (map) {
        var container = L.DomUtil.create('div', 'leaflet-control leaflet-bar'),
            link = L.DomUtil.create('a', '', container);

        link.href = '#';
        link.title = this.options.title;
        link.innerHTML = this.options.innerHTML;
        L.DomEvent.on(link, 'click', L.DomEvent.stop)
                  .on(link, 'click', function () {
                      window.LAYER = this.options.callback(this.options);
                  }, this);

        return container;
    }
});


//============================地图标注图标 单例公共对象====================== 
var iconObj = (function () {
    var instantiated;
    function init() {
        return {
            def: L.icon({
                iconUrl: 'resources/img/marker/mark_bs.png',
                iconSize: [30, 50],
                iconAnchor: [15, 25],
                popupAnchor: [0, -25],
                tooltipAnchor: [0, -25]
            }),
            poi: new L.Icon({
                iconUrl: 'resources/img/marker/marker_poi.png',
                iconSize: [26, 36],
                iconAnchor: [13, 27]
            }),
            start: L.icon({
                iconUrl: 'resources/img/marker/start.png',
                iconSize: [30, 42],
                iconAnchor: [15, 32],
                popupAnchor: [0, -24],
                tooltipAnchor: [0, -24]
            }),
            end: L.icon({
                iconUrl: 'resources/img/marker/end.png',
                iconSize: [30, 42],
                iconAnchor: [15, 32],
                popupAnchor: [0, -24],
                tooltipAnchor: [0, -24]
            })


        };
    }

    return {
        Instance: function () {
            if (!instantiated) {
                instantiated = init();
            }
            return instantiated;
        }
    };
})();