﻿/*
*  Simple navigation control that allows back and forward navigation through map's view history
*/

(function () {
    L.Control.NavBar = L.Control.extend({
        options: {
            position: 'topleft'
        },

        onAdd: function (map) {
            // Set options
            if (!this.options.center) {
                this.options.center = map.getCenter();
            }
            if (!this.options.zoom) {
                this.options.zoom = map.getZoom();
            }
            options = this.options;

            // Create toolbar
            var controlName = 'leaflet-control-navbar',
            container = L.DomUtil.create('div', controlName + ' leaflet-bar');

            // Add toolbar buttons
            this._homeButton = this._createButton('切换地图至全图默认区域', controlName + '-home', container, this._goHome);
            this._locateButton = this._createButton("定位至当前所在位置", controlName + '-locate', container, this._goLocate);
            this._locateButton = this._createButton("清除所有操作结果", controlName + '-clear', container, this._goClear);




            // Set intial view to home
            map.setView(options.center, options.zoom);
            return container;
        },

        _createButton: function (title, className, container, fn) {
            var link = L.DomUtil.create('a', className, container);
            link.href = '#';
            link.title = title;

            L.DomEvent
            .on(link, 'mousedown dblclick', L.DomEvent.stopPropagation)
            .on(link, 'click', L.DomEvent.stop)
            .on(link, 'click', fn, this)
            .on(link, 'click', this._refocusOnMap, this);

            return link;
        },


        _goHome: function () {
            if (this.options.bbox) {
                try {
                    this._map.fitBounds(this.options.bbox);
                } catch (err) {
                    this._map.setView(this.options.center, this.options.zoom); //Use default if invalid bbox input.
                }
            }
            this._map.setView(this.options.center, this.options.zoom);
        },

        _goLocate: function () {
            this._map.locate({ setView: true, maxZoom: 16 });
        },

        _goClear: function () {
            clearALL();
        },
         


        onRemove: function (map) {
            map.off('moveend', this._updateHistory, this);
        }
    });

    L.control.navbar = function (options) {
        return new L.Control.NavBar(options);
    };

})();
